#include <type.h>


inline char *
__type_str (char *name, type_t * type)
{
  char *str;

  assert (type);
  assert (type->name);
  assert (type->template >= 0);
  assert (type->sctype);

  asprintf (&str, "%s = [ name = %s, template = %d, sctype = %s ]", name,
	    type->name, type->template, type->sctype);
  return str;
}


inline int
type_size (type_t * type)
{

  assert (type);

  if (type->name)
    {
      if (!strcmp (type->name, "int"))
	return 32;
      if (!strcmp (type->name, "bool"))
	return 1;
      if (!strcmp (type->name, "sc_logic"))
	return 1;
      if (!strcmp (type->name, "sc_uint"))
	return type->template;
      if (!strcmp (type->name, "sc_int"))
	return type->template;
      if (!strcmp (type->name, "sc_biguint"))
	return type->template;
      if (!strcmp (type->name, "sc_bigint"))
	return type->template;
      if (!strcmp (type->name, "sc_lv"))
	return type->template;
      if (!strcmp (type->name, "void"))
	return 0;
    }

  //panic ("unknow type name %s (%s)", type->name, type_str(type));

  return 32;
}

inline int
type_signed (type_t * type)
{

  assert (type);

  if (type->name)
    {
      if (!strcmp (type->name, "int"))
	return true;
      if (!strcmp (type->name, "bool"))
	return false;
      if (!strcmp (type->name, "sc_logic"))
	return false;
      if (!strcmp (type->name, "sc_uint"))
	return false;
      if (!strcmp (type->name, "sc_int"))
	return true;
      if (!strcmp (type->name, "sc_biguint"))
	return false;
      if (!strcmp (type->name, "sc_bigint"))
	return true;
      if (!strcmp (type->name, "sc_lv"))
	return false;
    }

  return false;
}


static int
ast_value (Agnode_t * ast)
{
  //print_ast( ast );

  int value = 0;
  char *node_str = get_fsattr (ast);

  str_case (node_str, cUnaryOperator)
  {
    char *operator_str = get_namedattr (ast, "op");
    str_case (operator_str, ())
    {
      value = ast_value (get_fschild (ast));
    }
    str_elseif (operator_str, +)
    {
      value = ast_value (get_fschild (ast));
    }
    str_elseif (operator_str, -)
    {
      value = -ast_value (get_fschild (ast));
    }
    str_else
    {
      panic ("can't assign a value to %s unary operator", operator_str);
    }
    free (operator_str);
  }
  str_elseif (node_str, cBinaryOperator)
  {

    //print_ast( ast );

    char *operator_str = get_namedattr (ast, "op");
    str_case (operator_str, +)
    {
      value = ast_value (get_fschild (ast)) + ast_value (get_scchild (ast));
    }
    str_elseif (operator_str, -)
    {
      value = ast_value (get_fschild (ast)) - ast_value (get_scchild (ast));
    }
    str_elseif (operator_str, *)
    {
      value = ast_value (get_fschild (ast)) * ast_value (get_scchild (ast));
    }
    str_elseif (operator_str, /)
    {
      value = ast_value (get_fschild (ast)) / ast_value (get_scchild (ast));
    }
    str_elseif (operator_str, <<)
    {
      value = ast_value (get_fschild (ast)) << ast_value (get_scchild (ast));
    }
    str_elseif (operator_str, >>)
    {
      value = ast_value (get_fschild (ast)) >> ast_value (get_scchild (ast));
    }
    str_else
    {
      panic ("can't assign a value to %s binary operator", operator_str);
    }
    free (operator_str);
  }
  str_elseif (node_str, cLiteral)
  {
    char *operand_str = get_namedattr (ast, "value");
    assert (operand_str);
    value = atoi (operand_str);
    free (operand_str);
  }
  str_else
  {
    panic ("can't assign a value to a %s node", node_str);
  }

  free (node_str);

  return value;
}

static inline void
type_load (type_t * type, Agnode_t * root)
{

  Agnode_t *arg;

  assert (root);

  //t->sctype = t->name;

  char *node_str = get_fsattr (root);

  if (!strcmp (node_str, "cArrayType"))
    {
      //type->name = get_namedattr( root, "name" );
      //printf( "this is an array !\n" );
      type->array.n++;
      type->array.dim =
	(int *) realloc (type->array.dim, sizeof (int) * type->array.n);
      type->array.dim[type->array.n - 1] = ast_value (get_scchild (root));
      //print_ast( get_scchild( root ) );

      Agnode_t *n;

      n = resolve_path (root, 1, "cInstantiatedForwardType");

      if (!n)
	n = resolve_path (root, 1, "cBuiltInType");

      if (!n)
	{
	  panic ("can't resolve array type!");
	}

      type_load (type, n);

      free (node_str);

      return;
    }

  if (!strcmp (node_str, "cBuiltInType"))
    {

      type->name = get_namedattr (root, "name");
      assert (type->name);

      return;
    }

  if (!strcmp (node_str, "cForwardType"))
    {

      type->name = get_namedattr (root, "name");
      type->sctype = type->name;
      assert (type->name);

      return;
    }

  free (node_str);

  arg = resolve_path (root, 2, "cTemplateArgumentList", "cBuiltInType");

  if (arg)
    {
      type->name = get_namedattr (arg, "name");
      assert (type->name);
      return;
    }

  arg = resolve_path (root, 2, "cTemplateArgumentList", "cForwardType");
  if (arg)
    {
      char *str = get_namedattr (arg, "name");
      assert (str);
      type->name = str;
      return;
    }

  // handles variables
  arg = resolve_path (root, 2, "cTemplateArgumentList", "cLiteral");
  if (arg)
    {
      char *str = get_namedattr (root, "name");
      assert (str);
      type->name = str;
      str = get_namedattr (arg, "value");
      type->template = atoi (str);
      return;
    }

  // handles signals
  arg = resolve_path (root, 2, "cTemplateArgumentList", "cNameReference");
  if (arg)
    {
      char *str = get_namedattr (arg, "name");
      assert (str);
      type->name = str;
      assert (arg);
      //print_ast( arg );
      Agnode_t *value =
	resolve_path (arg, 2, "cTemplateArgumentList", "cLiteral");
      int val = 0;

      if (value)
	{
	  str = get_namedattr (value, "value");
	  assert (str);
	  val = atoi (str);
	}
      else if (!value)
	{

	  value =
	    resolve_path (arg, 2, "cTemplateArgumentList", "cUnaryOperator");

	  if (!value)
	    value =
	      resolve_path (arg, 2, "cTemplateArgumentList",
			    "cBinaryOperator");

	  if (value)
	    {
	      val = ast_value (value);
	      if (val <= 0)
		{
		  print_ast (value);
		  fprintf (stderr, "value = %d\n", val);
		}
	    }
	  else
	    {
	      print_ast (root);
	      panic ("can't find a template pattern");
	    }
	}

      assert (arg);
      assert (val > 0);

      type->template = val;

      type->sctype = get_namedattr (root, "name");
      assert (type->sctype);

      return;
    }

  arg = resolve_path (root, 2, "cTemplateArgumentList", "cBinaryOperator");
  if (arg)
    {
      type->name = get_namedattr (root, "name");
      assert (type->name);
      type->template = ast_value (arg);
      type->sctype = type->name;
      return;
    }

  arg = resolve_path (root, 2, "cTemplateArgumentList", "cUnaryOperator");
  if (arg)
    {
      type->name = get_namedattr (root, "name");
      assert (type->name);
      type->template = ast_value (arg);
      type->sctype = type->name;
      return;
    }

  print_ast (root);
  panic ("can't load type !");

  return;

}

inline type_t *
type_new (Agnode_t * root)
{

  assert (root);

  type_t *t = (type_t *) malloc (sizeof (type_t));
  assert (t);

  t->implementation = root;
  t->name = NULL;
  t->sctype = NULL;
  t->structdef = NULL;
  t->template = 0;

  t->array.n = 0;
  t->array.dim = NULL;

  char *str = get_namedattr (root, "name");
  if (str == NULL)
    {
      print_ast (root);
    }
  assert (str);
  t->sctype = str;

  assert (t->sctype);

  type_load (t, root);

  return t;
}

inline void
type_free (type_t * t)
{
  assert (t);
  free (t);
  return;
}
